@@ -2,11 +2,11 @@
 use std::sync::atomic::Ordering::{Relaxed, Release};
 use std::sync::atomic::{AtomicBool, AtomicUsize};
 use std::task::{Context, Poll, Waker};
-use std::{cell::Cell, cell::RefCell, fmt, future::Future, mem, pin::Pin, rc::Rc};
+use std::{cell::Cell, cell::RefCell, fmt, future::Future, mem, pin::Pin, ptr, rc::Rc};
 
 use futures_core::task::__internal::AtomicWaker;
 
-use crate::BytesMut;
+use crate::{BytesMut, BytesVec};
 
 pub struct Pool {
     idx: Cell<usize>,
@@ -21,8 +21,8 @@ pub struct PoolId(u8);
 
 #[derive(Copy, Clone)]
 pub struct BufParams {
-    pub high: u16,
-    pub low: u16,
+    pub high: u32,
+    pub low: u32,
 }
 
 bitflags::bitflags! {
@@ -50,9 +50,9 @@ struct MemoryPool {
 
     // io read/write cache and params
     read_wm: Cell<BufParams>,
-    read_cache: RefCell<Vec<BytesMut>>,
+    read_cache: RefCell<Vec<BytesVec>>,
     write_wm: Cell<BufParams>,
-    write_cache: RefCell<Vec<BytesMut>>,
+    write_cache: RefCell<Vec<BytesVec>>,
 
     spawn: RefCell<Option<Rc<dyn Fn(Pin<Box<dyn Future<Output = ()>>>)>>>,
 }
@@ -99,14 +99,14 @@ impl PoolId {
 
     #[doc(hidden)]
     #[inline]
-    pub fn set_read_params(self, h: u16, l: u16) -> Self {
+    pub fn set_read_params(self, h: u32, l: u32) -> Self {
         self.pool_ref().set_read_params(h, l);
         self
     }
 
     #[doc(hidden)]
     #[inline]
-    pub fn set_write_params(self, h: u16, l: u16) -> Self {
+    pub fn set_write_params(self, h: u32, l: u32) -> Self {
         self.pool_ref().set_write_params(h, l);
         self
     }
@@ -190,10 +190,21 @@ impl PoolRef {
         buf.move_to_pool(self);
     }
 
+    #[inline]
+    pub fn move_vec_in(self, buf: &mut BytesVec) {
+        buf.move_to_pool(self);
+    }
+
     #[inline]
     /// Creates a new `BytesMut` with the specified capacity.
     pub fn buf_with_capacity(self, cap: usize) -> BytesMut {
-        BytesMut::with_capacity_in_priv(cap, self)
+        BytesMut::with_capacity_in(cap, self)
+    }
+
+    #[inline]
+    /// Creates a new `BytesVec` with the specified capacity.
+    pub fn vec_with_capacity(self, cap: usize) -> BytesVec {
+        BytesVec::with_capacity_in(cap, self)
     }
 
     #[doc(hidden)]
@@ -246,7 +257,7 @@ impl PoolRef {
 
     #[doc(hidden)]
     #[inline]
-    pub fn set_read_params(self, h: u16, l: u16) -> Self {
+    pub fn set_read_params(self, h: u32, l: u32) -> Self {
         assert!(l < h);
         self.0.read_wm.set(BufParams { high: h, low: l });
         self
@@ -266,7 +277,7 @@ impl PoolRef {
 
     #[doc(hidden)]
     #[inline]
-    pub fn set_write_params(self, h: u16, l: u16) -> Self {
+    pub fn set_write_params(self, h: u32, l: u32) -> Self {
         assert!(l < h);
         self.0.write_wm.set(BufParams { high: h, low: l });
         self
@@ -274,18 +285,18 @@ impl PoolRef {
 
     #[doc(hidden)]
     #[inline]
-    pub fn get_read_buf(self) -> BytesMut {
+    pub fn get_read_buf(self) -> BytesVec {
         if let Some(buf) = self.0.read_cache.borrow_mut().pop() {
             buf
         } else {
-            BytesMut::with_capacity_in_priv(self.0.read_wm.get().high as usize, self)
+            BytesVec::with_capacity_in(self.0.read_wm.get().high as usize, self)
         }
     }
 
     #[doc(hidden)]
     #[inline]
     /// Release read buffer, buf must be allocated from this pool
-    pub fn release_read_buf(self, mut buf: BytesMut) {
+    pub fn release_read_buf(self, mut buf: BytesVec) {
         let cap = buf.capacity();
         let (hw, lw) = self.0.read_wm.get().unpack();
         if cap > lw && cap <= hw {
@@ -299,18 +310,18 @@ impl PoolRef {
 
     #[doc(hidden)]
     #[inline]
-    pub fn get_write_buf(self) -> BytesMut {
+    pub fn get_write_buf(self) -> BytesVec {
         if let Some(buf) = self.0.write_cache.borrow_mut().pop() {
             buf
         } else {
-            BytesMut::with_capacity_in_priv(self.0.write_wm.get().high as usize, self)
+            BytesVec::with_capacity_in(self.0.write_wm.get().high as usize, self)
         }
     }
 
     #[doc(hidden)]
     #[inline]
     /// Release write buffer, buf must be allocated from this pool
-    pub fn release_write_buf(self, mut buf: BytesMut) {
+    pub fn release_write_buf(self, mut buf: BytesVec) {
         let cap = buf.capacity();
         let (hw, lw) = self.0.write_wm.get().unpack();
         if cap > lw && cap <= hw {
@@ -378,6 +389,14 @@ impl fmt::Debug for PoolRef {
     }
 }
 
+impl Eq for PoolRef {}
+
+impl PartialEq for PoolRef {
+    fn eq(&self, other: &PoolRef) -> bool {
+        ptr::eq(&self.0, &other.0)
+    }
+}
+
 impl MemoryPool {
     fn create(id: PoolId) -> &'static MemoryPool {
         Box::leak(Box::new(MemoryPool {
@@ -459,7 +478,7 @@ impl fmt::Debug for Pool {
         f.debug_struct("Pool")
             .field("id", &self.id().0)
             .field("allocated", &self.inner.size.load(Relaxed))
-            .field("ready", &!self.is_pending())
+            .field("ready", &self.is_ready())
             .finish()
     }
 }
@@ -472,17 +491,25 @@ impl Pool {
     }
 
     #[inline]
-    /// Check if pool is pedning
-    pub fn is_pending(&self) -> bool {
+    /// Check if pool is ready
+    pub fn is_ready(&self) -> bool {
         let idx = self.idx.get();
         if idx > 0 {
             if let Some(Entry::Occupied(_)) =
                 self.inner.waiters.borrow().entries.get(idx - 1)
             {
-                return true;
+                return false;
             }
         }
-        false
+        true
+    }
+
+    #[inline]
+    #[doc(hidden)]
+    #[deprecated]
+    /// Check if pool is pedning
+    pub fn is_pending(&self) -> bool {
+        !self.is_ready()
     }
 
     #[doc(hidden)]
